(function(window,undefined)
{
	var SLIDE_WIDTH = 1024, SLIDE_HEIGHT = 768;
	
	/**
	 * Designer
	 * 
	 * @alias Designer
	 */
	var Designer = window.Designer = 
	{
		/**
		 * @type {Designer.Context} 
		 */
		c : {},		//Contexts
		emptyFunction : function(){},
		emptyFalseFunuction : function(){return false;},
		
		/**
		 * 填充默认参数
		 * 
		 * @alias	Designer.fillParams
		 * 
		 * @param	{Object}	options			参数
		 * @param	{Object}	defaultValue	默认参数
		 */
		fillParams: function(options, defaultValue)
		{
			if(options == undefined)
				return defaultValue;
			
			var ret = jQuery.extend({}, options);
			
			for(var key in defaultValue)
			{
				if(ret[key] == undefined)
					ret[key] = defaultValue[key];
			}
			
			return ret;
		},
		
		/**
		 * 判断一个节点是否是另一个节点的父节点
		 * http://www.cnblogs.com/rubylouvre/archive/2011/05/30/1583523.html
		 * 
		 * @alias	Designer.contains
		 * 
		 * @param	{Node}	root	父节点
		 * @param	{Node}	el		子节点
		 */
		contains: function(root, el)
		{
			if (root.compareDocumentPosition)
				return root === el || !!(root.compareDocumentPosition(el) & 16);
			
			if (root.contains && el.nodeType === 1)
				return root.contains(el) && root !== el;
			
			while ((el = el.parentNode))
				if (el === root) return true;
			
			return false;
		},		
		
		/**
		 * 全局事件
		 *
		 * @alias	Designer.Events
		 */
		Events :
		{
			/**
			 * @private
			 */
			_hooks : {}, 
			/**
			 * @private
			 * @alias	Designer.Events.init
			 */
			init : function()
			{
				var self = Designer.Events;

				Designer.context.addListener('destroy', 'Designer.GLOBAL_HOOKS', function(id)
				{
					for(var key in self._hooks)
					{
						if(self._hooks[key][id] != undefined)
							self.unhook(key, id);
					}
				});
			},
			/**
			 * 添加一个事件钩子
			 *
			 * @alias	Designer.Events.hook
			 *
			 * @param	{String}	event		事件名
			 * @param	{String}	ns			命名空间
			 * @param	{Function}	callback	事件回调
			 */
			hook : function(event, ns, callback)
			{
				var self = Designer.Events;

				if(self._hooks[event] == undefined)
					self._hooks[event] = {};
				
				if(self._hooks[event][ns] != undefined)
					return false;

				self._hooks[event][ns] = callback;

				return true;
			},
			/**
			 * 撤销一个事件钩子
			 *
			 * @alias	Designer.Events.unhook
			 *
			 * @param	{String}	event	事件名
			 * @param	{String}	ns		命名空间
			 */
			unhook : function(event, ns)
			{
				var self = Designer.Events;

				if(self._hooks[event] == undefined || self._hooks[event][ns] == undefined)
					return false;
				
				delete self._hooks[event][ns];
			},
			/**
			 * 触发一个事件
			 *
			 * @alias	Designer.Events.trigger
			 *
			 * @param	{String}	event	事件名
			 */
			trigger : function(event, argu)
			{
				var dEvents = Designer.Events;
				
				//Not breaked
				if(dEvents._hooks[event] == undefined)
					return true;
				
				var callbacks = dEvents._hooks[event];
				
				if (argu == undefined) argu = [];
				argu.unshift(event);
				
				for (var key in callbacks)
				{
					if (typeof callbacks[key] == 'function')
					{
						if (callbacks[key].apply(Designer, argu) === false)
							return false;
					}
				}
			}
		},
		
		/**
		 * 调试相关
		 *
		 * @alias	Designer.Debug
		 */
		Debug :
		{
			/**
			 * 执行调试输出语句
			 *
			 * @alias	Designer.Debug._execute
			 * @private
			 * @internal
			 *
			 * @param	{String}	namespace	命名空间
			 * @param	{Object}	data		输出内容
			 */
			_execute : function(namespace, data)
			{
				var func = this;

				if(!Designer.DebugEnabled || func == undefined)
					return;
				
				func.call(console, ('[{time}] {ns} > ').format(
				{
					time : new Date().formatTime(),
					ns : namespace
				}), data.toString());
			},
			/**
			 * 在控制台输出一条信息
			 *
			 * @alias	Designer.Debug.log
			 */
			log : function(namespace, data)
			{
				Designer.Debug._execute.apply(console.log, arguments);
			},
			/**
			 * 在控制台输出一条警告
			 *
			 * @alias	Designer.Debug.warn
			 */
			warn : function(namespace, data)
			{
				Designer.Debug._execute.apply(console.warn, arguments);
			},
			/**
			 * 在控制台输出一条错误
			 *
			 * @alias	Designer.Debug.error
			 */
			error : function(namespace, data)
			{
				Designer.Debug._execute.apply(console.error, arguments);
			}
		},
		
		/**
		 * 标识符相关
		 *
		 * @alias	Designer.Id
		 */
		Id :
		{
			counter : 0,
			
			/**
			 * 生成一个随机的ID
			 *
			 * @alias	Designer.Id.random
			 * @return	{String}
			 */
			random : function()
			{
				Designer.Id.counter++;
				return 'ID_' + (new Date()).valueOf().toString(16) + '_' + Designer.Id.counter.toString(16);
			}
		},
		
		/**
		 * 对话框相关
		 * 
		 * @alias	Designer.Dialog
		 */
		Dialog :
		{
			_$e : null,
			_$c : null,
			_shown : false,
			_data : null,
			/**
			 * 初始化
			 * 
			 * @alias	Designer.Dialog._init 
			 */
			_init : function()
			{
				var dDialog = Designer.Dialog;

				if(dDialog._$e !== null)
					return;

				dDialog._$e = $('#dialogContainer');//.click(dDialog.tryCancel);
				dDialog._$c = dDialog._$e.find('#dialog').click(dDialog._cancelBubble);
			},
			_cancelBubble : function(event)
			{
				event.stopPropagation();
			},
			/**
			 * 创建对话框
			 * 
			 * @alias	Designer.Dialog.create
			 * @remarks	options:
			 * 			{
			 * 				cancelable: false,
			 * 				buttons: [{c: content, e: extra class, onClick: on click}],
			 * 				content
			 * 			}
			 *  
			 */
			create : function(options)
			{
				var dDialog = Designer.Dialog;
				if(dDialog._$e === null)
					dDialog._init();
				
				options = Designer.fillParams(options,
				{
					cancelable : false,
					buttons : [],
					content : ''
				});

				if(dDialog._shown)
					return;

				dDialog._data = options;
				dDialog._$c.find('.content').empty().append(options.content);

				var $toolstrip = dDialog._$c.find('.bar').empty();
				for(var key in options.buttons)
				{
					var c = options.buttons[key].onClick || Designer.emptyFunction;
					$('<input type="button" class="button"></input>').val(options.buttons[key].c).click(c).addClass(options.buttons[key].e).appendTo($toolstrip);
				}
				
				dDialog._$e.css('display', 'block');
				dDialog._$e.fadeTo(200, 1);
				
				dDialog._shown = true;
			},
			startLoading : function()
			{
				var $bar = Designer.Dialog._$c.find('.bar');
				if($bar.find('.loading').length > 0)
					return;

				$('<span class="loading"></span>').prependTo($bar);
			},
			endLoading : function()
			{
				Designer.Dialog._$c.find('.bar .loading').remove();
			},
			close : function(immediate, callback)
			{
				var dDialog = Designer.Dialog;

				if(!dDialog._shown)
					return;
				
				if (typeof callback != 'function') callback = Designer.emptyFunction;
				
				dDialog.endLoading();
				
				if (immediate === true)
				{
					dDialog._$e.hide();
					dDialog._$c.find('.content').empty();
					dDialog._$c.find('.bar').empty();
					dDialog._shown = false;
					
					callback();
				}
				else
				{
					dDialog._$e.fadeTo(500, 0, function()
					{
						$(this).css('display', 'none');
						dDialog._$c.find('.content').empty();
						dDialog._$c.find('.bar').empty();
						dDialog._shown = false;
						
						callback();
					});
				}
			},
			tryCancel : function()
			{
				var dDialog = Designer.Dialog;
				if (dDialog._shown && dDialog._data != undefined && dDialog._data.cancelable)
					dDialog.close();
			}
		},
		
		_InfoBase :
		{
			/**
			 * 初始化
			 *
			 * @alias	Designer._InfoBase._init
			 */
			_init : function()
			{
				var self = Designer._InfoBase;

				self._$e.hover(function()
				{
					Designer.c['d'].destroyDelayer(self._ns + '.hide');
				}, function()
				{
					Designer.c['d'].newDelayer(self._ns + '.hide', 3000, function()
					{
						self.hide();
					});
				});
			},
			/**
			 * 显示提示
			 *
			 * @alias	Designer._InfoBase._show
			 *
			 * @param	{DOM, String}		content		用于显示的提示文字或DOM
			 * @param	{Number}			[hideMs]	多少毫秒后隐藏，默认5000（5秒）
			 * @param	{Number}			[delayMs]	多少毫秒后显示，默认为0（立即显示）
			 */
			_show : function(content, hideMs, delayMs)
			{
				var self = Designer._InfoBase;
				if(self._$e === null)
					self._init();
				hideMs = hideMs || 5000;
				
				//Delay show
				Designer.c['d'].destroyDelayer(self._ns + '.show');
				if(delayMs != undefined && delayMs > 0)
				{
					Designer.c['d'].newDelayer(self._ns + '.show', delayMs, function()
					{
						self.show(content, hideMs);
					});
					return;
				}

				//如果是文字则包裹<span/>，如果是jQueryNode则直接insert
				if(!content instanceof jQuery)
					content = $('<span/>').html(content);

				self._$c.empty().append(content);

				if(!self._shown)
				{
					self._shown = true;
					self._$e.stop(true).css('display', self._display);
					self._$f.fadeTo(200, 1);
					//Fade in container is incorrect : http://www.w3.org/TR/css3-color/#opacity
				}

				Designer.c['d'].destroyDelayer(self._ns + '.hide');
				Designer.c['d'].newDelayer(self._ns + '.hide', hideMs, function()
				{
					self.hide();
				});
			},
			/**
			 * 隐藏提示
			 *
			 * @alias	Designer._InfoBase._hide
			 */
			_hide : function(delayMs)
			{
				var self = Designer._InfoBase;

				Designer.c['d'].destroyDelayer(self._ns + '.show');

				if(delayMs)
				{
					Designer.c['d'].destroyDelayer(self._ns + '.hide');
					Designer.c['d'].newDelayer(self._ns + '.hide', delayMs, function()
					{
						self.hide();
					});
					return;
				}

				if(!self._shown)
					return;

				self._shown = false;
				
				self._$f.stop(true).fadeTo(500, 0, function()
				{
					self._$e.css('display', 'none');
				});
				
				Designer.c['d'].destroyDelayer(self._ns + '.hide');
			}
		},
		/**
		 * 提示信息
		 * 
		 * @alias	Designer.Info 
		 */
		Info :
		{
			_$e : null,
			_$c : null,
			_shown : false,
			_ns : 'Designer.Info',
			_display : 'block',
			_init : function()
			{
				var dInfo = Designer.Info;
				if(dInfo._$e !== null)
					return;
				dInfo._$e = $('#infoContainer');
				dInfo._$f = dInfo._$c = dInfo._$e.find('#info');
				
				Designer._InfoBase._init.apply(this);
			},
			show : function()
			{
				Designer._InfoBase._show.apply(this, arguments);
			},
			hide : function()
			{
				Designer._InfoBase._hide.apply(this, arguments);
			}
		}
	};
	
	/**
	 * 页面上下文管理
	 *
	 * @alias	Designer.context
	 */
	Designer.context =
	{
		/**
		 * @private
		 */
		_listener :
		{
			alloc :
			{
			},
			destroy :
			{
			}
		},

		/**
		 * 添加事件监听
		 *
		 * @alias	Designer.context.addListener
		 * @return	{Boolean}	是否成功
		 *
		 * @param	{String}	type		监听类型，可用的为'alloc'，'destroy'
		 * @param	{String}	id			标识符
		 * @param	{Function}	callback	回调
		 */
		addListener : function(type, id, callback)
		{
			var self = Designer.context;

			if(type !== 'alloc' && type !== 'destroy')
				return false;
			if(id == undefined)
				return false;
			if(self._listener[type][id] != undefined)
				return false;

			self._listener[type][id] = callback;

			return true;
		},
		/**
		 * 移除事件监听
		 *
		 * @alias	Designer.context.removeListener
		 * @return	{Boolean}	是否成功
		 *
		 * @param	{String}	type		监听类型
		 * @param	{String}	id			标识符
		 */
		removeListener : function(type, id)
		{
			var self = Designer.context;

			if(type !== 'alloc' && type !== 'destroy')
				return false;
			if(id == undefined)
				return false;
			if(self._listener[type][id] == undefined)
				return false;
			delete self._listener[type][id];

			return true;
		},
		/**
		 * 根据id创建一个上下文，之后可以通过 Designer.c[id] 获取对这个上下文的引用
		 *
		 * @alias	Designer.context.alloc
		 *
		 * @param	{String}	id		标识符
		 */
		alloc : function(id)
		{
			var self = Designer.context;

			if(Designer.c[id] != undefined)
			{
				Designer.Debug.error('Designer.context.alloc', ('id[{id}] already existed.').format(
				{
					id : id
				}));
				return;
			}

			for(var key in self._listener.alloc)self._listener.alloc[key](id);

			Designer.c[id] = new Designer.Context(id);
		},
		/**
		 * 销毁一个上下文，并销毁该上下文的所有托管对象
		 *
		 * @alias	Designer.context.destroy
		 *
		 * @param	{String}	id		标识符
		 */
		destroy : function(id)
		{
			var self = Designer.context;

			if(Designer.c[id] == undefined)
				return;

			for(var key in self._listener.destroy)self._listener.destroy[key](id);

			Designer.c[id]._destroy();
			delete Designer.c[id];
		}
	};
	/**
	 * @classDescription	页面上下文对象
	 * @alias	Designer.Context
	 * @type	{Object}
	 * @return	{Designer.Context}
	 * @constructor
	 */
	Designer.Context = function(id)
	{
		var self = this;
		/**
		 * @private
		 */
		self._id = id.replace(/\./g, '_');
		/**
		 * @private
		 */
		self._self = 'Designer.context[' + self._id + ']';
		/**
		 * @private
		 */
		self._delayer =
		{
		};
		/**
		 * @private
		 */
		self._res =
		{
		};
		/**
		 * @private
		 */
		self._listener = [];

		return self;
	};
	Designer.Context.prototype = 
	{
		constructor : Designer,

		/**
		 * 销毁一个延迟回调
		 *
		 * @alias	Designer.Context._destroyNTimer
		 * @private
		 *
		 * @param	{String}	id		标识符
		 * @param	{Function}	func	TimeOut函数或Interval函数
		 */
		_destroyNTimer : function(id, func)
		{
			var self = this;

			if(self._delayer[id] == undefined)
				return self;

			func.call(window, self._delayer[id]);
			delete self._delayer[id];

			return self;
		},
		/**
		 * 创建一个timeout/interval
		 *
		 * @alias	Designer.Context._newNTimer
		 * @private
		 *
		 * @param	{String}	[id]		标识符
		 * @param	{Number}	delay		延迟时间(ms)
		 * @param	{Function}	callback	回调函数
		 * @param	{Function}	func		TimeOut函数或Interval函数
		 * @param	{Boolean}	[destroy]	是否销毁，默认为true
		 */
		_newNTimer : function(id, delay, callback, func, destroy)
		{
			var _id, _delay, _callback, _destroy;
			_destroy = destroy == undefined ? true : destroy;

			var self = this;

			if(( typeof delay) === 'function')
			{
				_id = Designer.Id.random();
				_delay = id;
				_callback = delay;
			}
			else
			{
				if(self._delayer[id] != undefined)
				{
					self.destroyDelayer(id);
					Designer.Debug.warn(self._self + '.newTimeoutInterval', ('id[{id}] already existed.').format(
					{
						id : id
					}));
					//return this;
				}
				_id = id;
				_delay = delay;
				_callback = callback;
			}

			self._delayer[_id] = func.call(window, function()
			{
				_callback();
				
				if(_destroy)
					self.destroyDelayer(_id);
			}, _delay);
			return self;
		},
		/**
		 * 添加一个事件监听
		 *
		 * @alias	Designer.Context.bindDOMListener
		 *
		 * @param	{Object}	$object		jQuery Object
		 * @param	{String}	event		事件
		 * @param	{Function}	callback	回调
		 */
		bindDOMListener : function($object, event, callback)
		{
			var self = this;

			var realEvent = event + '.' + self._id;

			if(!( $object instanceof jQuery))
				$object = $($object);

			self._listener.push(
			{
				obj : $object.get(),
				real : realEvent,
				callback : callback
			});

			$($object).bind(realEvent, callback);

			return self;
		},
		/**
		 * 撤销一个事件监听
		 *
		 * @alias	Designer.Context.unbindDOMListener
		 *
		 * @param	{Object}	$object		jQuery Object
		 * @param	{String}	event		事件
		 * @param	{Functino}	[Callback]	回调
		 */
		unbindDOMListener : function($object, event, callback)
		{
			var self = this;

			var realEvent = event + '.' + self._id;

			if(!( $object instanceof jQuery))
				$object = $($object);

			$($object).unbind(realEvent, callback);

			for(var key in self._listener)
			{
				if(self._listener[key].obj == $object.get() && self._listener[key].real === realEvent && (callback == undefined || self._listener[key].callback === callback))
				{
					delete self._listener[key];
					break;
				}
			}

			return self;
		},
		/**
		 * 创建一个延迟回调
		 *
		 * @alias	Designer.Context.newDelayer
		 *
		 * @param	{String}	[id]		标识符
		 * @param	{Number}	delay		延迟时间(ms)
		 * @param	{Function}	callback	回调
		 */
		newDelayer : function(id, delay, callback)
		{
			this._newNTimer(id, delay, callback, Designer._native_setTimeout);
			return this;
		},
		/**
		 * 创建一个间隔回调
		 *
		 * @alias	Designer.Context.newTimer
		 *
		 * @param	{String}	[id]		标识符
		 * @param	{Number}	delay		延迟时间(ms)
		 * @param	{Function}	callback	回调
		 * @param	{Boolean}	[immediate]	是否立即触发，默认为true
		 */
		newTimer : function(id, delay, callback, immediate)
		{
			if(( typeof delay) === 'function')
			{
				if(callback === true)
					delay();
			}
			else
			{
				if(immediate == true)
					callback();
			}

			this._newNTimer(id, delay, callback, Designer._native_setInterval, false);
			return this;
		},
		/**
		 * 销毁一个延迟回调
		 *
		 * @alias	Designer.Context.destroyDelayer
		 *
		 * @param	{String}	id		标识符
		 */
		destroyDelayer : function(id)
		{
			this._destroyNTimer(id, Designer._native_clearTimeout);
			return this;
		},
		/**
		 * 销毁一个间隔回调
		 *
		 * @alias	Designer.Context.destroyTimer
		 *
		 * @param	{String}	id		标识符
		 */
		destroyTimer : function(id)
		{
			this._destroyNTimer(id, Designer._native_clearInterval);
			return this;
		},
		/**
		 * 添加一个资源
		 *
		 * @alias	Designer.Context.addResource
		 *
		 * @param	{String}	id		标识符
		 * @param	{Object}	content	资源内容
		 */
		addResource : function(id, content)
		{
			var self = this;

			if(self._res[id] != undefined)
				return self;
			content = content ||
			{
			};
			self._res[id] = content;

			return self;
		},
		/**
		 * 返回一个资源
		 *
		 * @alias	Designer.Context.getResource
		 *
		 * @param	{String}	id		标识符
		 */
		getResource : function(id)
		{
			if(this._res[id] == undefined)
				return false;

			return this._res[id];
		},
		/**
		 * 更新一个资源
		 *
		 * @alias	Designer.Context.updateResource
		 *
		 * @param	{String}	id		标识符
		 * @param	{Object}	content	资源内容
		 */
		updateResource : function(id, content)
		{
			var self = this;

			if(self._res[id] == undefined)
				return self;

			self._res[id] = content;

			return self;
		},
		/**
		 * 销毁一个资源并触发回调
		 *
		 * @alias	Designer.Context.destroyResource
		 *
		 * @param	{String}	id		标识符
		 * @param	{Boolean}	[cb]	是否触发回调，默认为true
		 *
		 */
		destroyResource : function(id, cb)
		{
			var self = this;

			if(self._res[id] == undefined)
				return self;

			if(cb === true && self._res[id]._destroy != undefined)
				self._res[id]._destroy();
			delete self._res[id];

			return self;
		},
		/**
		 * 销毁所有上下文对象
		 *
		 * @alias	Designer.Context._destroy
		 * @private
		 */
		_destroy : function()
		{
			var self = this;

			for(var key in self._delayer)
			self.destroyDelayer(key);
			for(var key in self._res)
			self.destroyResource(key);

			try
			{
				for(var key in self._listener)
				$(self._listener[key].obj).unbind(self._listener[key].real);
			}
			catch(e)
			{
			}
			delete self._delayer;
			delete self._res;
			delete self._listener;
		}
	};
	
	//Timer托管
	Designer._native_setTimeout = window.setTimeout;
	Designer._native_setInterval = window.setInterval;
	Designer._native_clearTimeout = window.clearTimeout;
	Designer._native_clearInterval = window.clearInterval;
	Designer.currentNameSpace = 'unmanaged';
	
	window.setTimeout = function(callback, delay)
	{
		//野生setTimeout
		var ID = 'UnM_' + Designer.Id.random();
		Designer.c[Designer.currentNameSpace].newDelayer(ID, delay, callback);
		
		return ID;
	}
	
	window.setInterval = function(callback, interval)
	{
		//野生setInterval
		var ID = 'UnM_' + Designer.Id.random();
		Designer.c[Designer.currentNameSpace].newTimer(ID, interval, callback);
		
		return ID;
	}
	
	window.clearTimeout = function(ID)
	{
		Designer.c[Designer.currentNameSpace].destroyDelayer(ID);
	}
	
	window.clearInterval = function(ID)
	{
		Designer.c[Designer.currentNameSpace].destroyTimer(ID);
	}
	
	Designer.Events.init();
	
	//默认上下文
	Designer.context.alloc('d');
	Designer.context.alloc('unmanaged');
	
	//===========================
	
	/**
	 * (Event Handler : Designer.UI) 鼠标移动时触发
	 * @param {Object} event
	 */
	function cb_Designer_UI_resizer(event)
	{
		var dragObject = this;
		var dUI = Designer.UI;
		
		var v;
		
		//水平
		if (dragObject.hasClass('horizontal'))
		{
			v = event.clientX - dUI.DragClientX + dUI.DragObjOfsX;
			
			//min-width
			if (dUI.ResizeObj1.data('min-width') != undefined && v < dUI.ResizeObj1.data('min-width'))
				v = dUI.ResizeObj1.data('min-width');
			if (dUI.ResizeObj1.data('max-width') != undefined && v > dUI.ResizeObj1.data('max-width'))
				v = dUI.ResizeObj1.data('max-width');
			if (dUI.ResizeObj2.data('min-width') != undefined && (dUI.ResizePar.width() - v - dUI.ResizeSize) < dUI.ResizeObj2.data('min-width'))
				v = dUI.ResizePar.width() - dUI.ResizeObj2.data('min-width');
		}
		else
		{
			v = event.clientY - dUI.DragClientY + dUI.DragObjOfsY;
			
			//min-height
			if (dUI.ResizeObj1.data('min-height') != undefined && v < dUI.ResizeObj1.data('min-height'))
				v = dUI.ResizeObj1.data('min-height');
			if (dUI.ResizeObj1.data('max-height') != undefined && v > dUI.ResizeObj1.data('max-height'))
				v = dUI.ResizeObj1.data('max-height');
			if (dUI.ResizeObj2.data('min-height') != undefined && (dUI.ResizePar.height() - v - dUI.ResizeSize) < dUI.ResizeObj2.data('min-height'))
				v = dUI.ResizePar.height() - dUI.ResizeObj2.data('min-height');
		}
		
		return v;
	}
	
	/**
	 * (Event Handler : Designer.UI) resizer实现部分
	 * @param {Object} event
	 * @param {Object} v
	 * @param {Object} update	是否强制更新
	 */
	function cb_Designer_UI_resizer_realize(event, v, update)
	{
		var dragObject = this;
		var dUI = Designer.UI;
		
		if (dragObject.hasClass('horizontal'))
		{
			dragObject.css('left', v);
			
			if (dUI.ResizeRealtime || update)
			{
				dUI.ResizeObj1.css('width',v);
				dUI.ResizeObj2.css('left',v+dUI.ResizeSize);
			}
		}
		else
		{
			dragObject.css('top', v);
			
			if (dUI.ResizeRealtime || update)
			{
				dUI.ResizeObj1.css('height',v);
				dUI.ResizeObj2.css('top',v+dUI.ResizeSize);
			}
		}
	}
	
	/**
	 * 界面相关
	 * 
	 * @alias	Designer.UI 
	 */
	Designer.UI = 
	{
		DragInitialized: false,		//全局回调是否已初始化
		DragCallback: null,			//回调
		DragEndCallback: null,		//结束回调
		DragObject: null,			//拖动的内容
		DragObjOfsX: 0,				//拖动目标相对X
		DragObjOfsY: 0,				//拖动目标相对Y
		DragClientX: 0,				//开始时候X
		DragClientY: 0,				//开始时候Y
		DragOfsX: 0,				//开始时候相对X
		DragOfsY: 0,				//开始时候相对Y
		DragCursor: null,
		DragData: null,
		
		ResizeSize: 7,
		ResizePar: null,
		ResizeObj1: null,
		ResizeObj2: null,
		ResizeRealtime: null,
		
		/**
		 * 初始化用于拖动的全局事件监听
		 * 
		 * @alias	Designer.UI.dragInitialize 
		 */
		dragInitialize: function()
		{
			if (Designer.UI.DragInitialized) return;
			
			$(document).mousemove(function(event)
			{
				//左键按下
				if (event.which != 1) return;
				
				var dUI = Designer.UI;
				
				//拖动中
				if (dUI.DragObject !== null)
				{
					//触发回调
					if (typeof dUI.DragCallback == 'function')
						dUI.DragCallback.call(dUI.DragObject, event);
					
					event.preventDefault();
					return false;
				}
			});
			
			$(document).mouseup(function(event)
			{
				//鼠标松开
				var dUI = Designer.UI;
				
				if (dUI.DragObject !== null)
				{
					if (typeof dUI.DragCallback == 'function')
						dUI.DragCallback.call(dUI.DragObject, event, true);
					
					if (typeof dUI.DragEndCallback == 'function')
						dUI.DragEndCallback.call(dUI.DragObject, event);
				}
				
				$('body').css('cursor', 'auto');
				dUI.DragObject = null;
			});
			
			Designer.UI.DragInitialized = true;
		},
		
		/*
		 * 开始拖曳
		 */
		startDrag: function(dragObject, event)
		{
			var dUI = Designer.UI;
			
			dragObject = $(dragObject);
			
			dUI.DragClientX = event.clientX;
			dUI.DragClientY = event.clientY;
			dUI.DragOfsX = event.offsetX;
			dUI.DragOfsY = event.offsetY;
			dUI.DragObjOfsX = dragObject.position().left;
			dUI.DragObjOfsY = dragObject.position().top;
			dUI.DragObject = dragObject;
			
			dUI.DragCursor = dUI.DragCursor || 'auto';
			$('body').css('cursor', dUI.DragCursor);
			
			//Prevent default will block blur event
			$(document.activeElement).blur();
			
			event.preventDefault();
		},
		
		/**
		 * 使两栏可以改变大小 
		 * 
		 * @alias Designer.UI.panelResizable
		 * 
		 * @param {Object} $panel	两栏的父元素
		 * @param {Object} options
		 */
		panelResizable: function($panel, options)
		{
			if ($panel.children().length < 2) return false;
			
			var dUI = Designer.UI;
			
			options = Designer.fillParams(options,
			{
				horizontal: true,
				realtime: true,
				onResizing: Designer.emptyFunction,
				onStart: Designer.emptyFunction,
				onEnd: Designer.emptyFunction
			});
			
			//初始化Drag
			if (!dUI.DragInitialized)
				dUI.dragInitialize();
			
			var $resize = $('<div class="panel_resizer"></div>');
			$resize.appendTo($panel);
			
			if (options.horizontal)
				var ResizeSize = $resize.outerWidth(true);
			else
				var ResizeSize = $resize.outerHeight(true);
			
			var ResizeObj1 = $panel.children('.resizer1');
			var ResizeObj2 = $panel.children('.resizer2');
			
			$resize.mousedown(function(event)
			{
				var dUI = Designer.UI;
				dUI.ResizeRealtime = options.realtime;
				dUI.ResizePar = $panel;
				dUI.ResizeObj1 = ResizeObj1;
				dUI.ResizeObj2 = ResizeObj2;
				dUI.DragCursor = 'col-resize';
				dUI.ResizeSize = ResizeSize;
				
				dUI.DragEndCallback = options.onEnd;
				dUI.DragCallback = function(event, update)
				{
					var v = cb_Designer_UI_resizer.call(this, event);
					if (options.onResizing.call(this, event, v) !== false)
						cb_Designer_UI_resizer_realize.call(this, event, v, update);
				}
				dUI.startDrag(this, event);
				options.onStart.call(this, event);
			});
			
			var v;
			
			//设置Resizer位置
			if (options.horizontal)
			{
				v = ResizeObj1.width();
				$resize.addClass('horizontal');
				$resize.css('left', v);
				ResizeObj2.css('left', v+ResizeSize);
			}
			else
			{
				v = ResizeObj1.height();
				$resize.addClass('vertical');
				$resize.css('top', v);
				ResizeObj2.css('top', v+ResizeSize);
			}
			
			return true;
		},
		
		/**
		 * 创建单选元素
		 * 
		 * @alias Designer.UI.initSingleSelectElements
		 * 
		 * @param {Object} $elements	单选元素集合
		 */
		initSingleSelectElements: function($elements)
		{
			$elements.click(function()
			{
				var self = $(this);
				if (self.hasClass('selected')) return;
				
				$elements.removeClass('selected');
				self.addClass('selected');
			});
		},
		
		/**
		 * 创建多选元素
		 * 
		 * @alias Designer.UI.initMultipleSelectElements 
		 * @param {Object} $elements	多选元素集合
		 */
		initMultipleSelectElements: function($elements)
		{
			$elements.click(function()
			{
				$(this).toggleClass('selected');
			});
		},
		
		/**
		 * 创建交互性Label 
		 * 
		 * @alias Designer.UI.initExtendLabel
		 */
		initExtendLabel: function($labels, options)
		{
			$labels.each(function()
			{
				var $label = $(this);
				
				$label.children('.c')
				.mousedown(eh_Designer_UI_exlabel_content_mousedown)
				.mouseup(eh_Designer_UI_exlabel_content_mouseup);
				
				options = Designer.fillParams(options,
				{
					onChange: Designer.emptyFunction,
					onEditBegin: Designer.emptyFunction,
					onEditEnd: Designer.emptyFunction
				});
				
				$label.data('callbacks', options);
				
				var settings = $label.attr('data-set');
				if (settings != null)
					settings = settings.replace(/\'/g,'"');
				
				try
				{
					var obj = JSON.parse(settings);
					$label.data('settings', obj);
				}
				catch(e)
				{
				}
			});
		},
		
		//TODO: EDIT
		bindContextMenu: function($target, options)
		{
			if ($target == undefined || options == undefined) return false;
			if (options.$menu == undefined) return false;
			
			options.callback = options.callback || Designer.emptyFunction;
			options.$menu.hide();
			options.$menu.children('li')
			.mousemove(function()
			{
				if (!$(this).hasClass('hover'))
				{
					options.$menu.children('li').removeClass('hover');
					$(this).addClass('hover');
					options.$menu.data('current-index', $(this).data('index'));
				}
			})
			.hover(null, function()
			{
				options.$menu.children('li').removeClass('hover');
			})
			.mousedown(function()
			{
				return false;
			})
			.click(function(event)
			{
				options.callback($(this).attr('data-action'), this);
				options.$menu.fadeOut(200);
			})
			.each(function(idx)
			{
				$(this).data('index', idx);
			});
			
			options.$menu
			.keydown(function(event){
				switch(event.which)
				{
					case 32:	//space
					case 13:	//enter
						$(this).children('li').eq($(this).data('current-index')).click();
						break;
					case 40:	//down
						var idx = $(this).data('current-index');
						if (idx >= $(this).children('li').length - 1) idx = -1;
						
						$(this).data('current-index', ++idx);
						$(this).children('li').removeClass('hover').eq(idx).addClass('hover');
						
						break;
					case 38:	//up
						var idx = $(this).data('current-index');
						if (idx <= 0) idx = $(this).children('li').length;
						
						$(this).data('current-index', --idx);
						$(this).children('li').removeClass('hover').eq(idx).addClass('hover');
						
						break;
					case 18:	//Alt
						$(this).fadeOut(200);
						
						break;
				}
			});
			
			if ($(document).data('context-menu-handler-installed') == undefined)
			{
				$(document).data('context-menu-handler-installed', true);
				$(document).mousedown(function(event)
				{
					$('body>.menustrip').fadeOut(200);
				});
			}
			
			$target.mousedown(function(event)
			{
				//Right click
				if (event.button !== 2) return;
				
				options.$menu.children('li').removeClass('hover');
				
				options.$menu
					.css({left: event.pageX, top: event.pageY})
					.data('current-index', -1)
					.stop(true,true).fadeIn(100).focus();
				
				event.stopPropagation();
				return false;
			});
			
			options.$menu.bind('contextmenu', function(){return false;});
			$target.bind('contextmenu', function(){return false;});
		}
	};
	
	function eh_Designer_UI_exlabel_content_dragEnd(event)
	{
		var dUI = Designer.UI;
		
		if (event.clientX == dUI.DragClientX && event.clientY == dUI.DragClientY) return;
		
		var $label = dUI.DragData.label;
		$label.data('callbacks').onEditEnd.call($label);
	}
	
	function eh_Designer_UI_exlabel_content_dragging(event)
	{
		var dUI = Designer.UI;
		var $label = dUI.DragData.label;
		var settings = dUI.DragData.settings;
		
		if (event.clientX == dUI.DragClientX && event.clientY == dUI.DragClientY) return;
		$label.removeData('valid_mouseup');
		
		var d_step = settings.step || 1;
		var d_floor = settings.floor || false;
		var d_max = settings.max;
		var d_min = settings.min;
		var ordValue = dUI.DragData.original;
		var d_delta = event.clientX - dUI.DragClientX;
		var d_value = ordValue + Math.floor(d_delta / 2) * d_step;
		
		if (d_floor)
			d_value = Math.floor(d_value);
		else
			d_value = d_value.toFixed(2);
		
		if (d_max != undefined && d_value > d_max)
			d_value = d_max;
		
		if (d_min != undefined && d_value < d_min)
			d_value = d_min;
		
		if (d_value != $label.children('.c').text())
		{
			$label.children('.c').text(d_value);
			$label.data('callbacks').onChange.call($label, d_value);
		}
	};
	
	function eh_Designer_UI_exlabel_content_mousedown(event)
	{
		var $content = $(this);
		var $label = $(this).parent();
		
		$label.data('valid_mouseup', true);
		$label.addClass('active');
		
		$label.data('callbacks').onEditBegin.call($label);
		
		if ($label.data('settings') != undefined)
		{
			var dUI = Designer.UI;
			dUI.DragCursor = 'ew-resize';
			dUI.DragData = {settings: $label.data('settings'), label: $label, original: parseFloat($content.text())};
			
			dUI.DragEndCallback = eh_Designer_UI_exlabel_content_dragEnd;
			dUI.DragCallback = eh_Designer_UI_exlabel_content_dragging;
			dUI.startDrag($('#stage'), event);
		}
		
		$(document.activeElement).blur();
		event.preventDefault();
	}
	
	function eh_Designer_UI_exlabel_content_mouseup()
	{
		var $content = $(this);
		var $label = $(this).parent();
		
		if ($label.data('valid_mouseup') == undefined) return;
		$label.removeData('valid_mouseup');
		
		var $input = $('<input type="text">');
		
		$content.hide();
		$input.val($content.text());
		$input
			.blur(eh_Designer_UI_exlabel_input_blur)
			.keydown(eh_Designer_UI_exlabel_input_keydown);
		
		$input.appendTo($label).focus();
	}
	
	function eh_Designer_UI_exlabel_input_keydown(event)
	{
		if (event.which == 13)			//Enter
			eh_Designer_UI_exlabel_input_blur.call(this);
		else if (event.which == 27)	//ESC
			eh_Designer_UI_exlabel_input_blur.call(this, false);
	}
	
	function eh_Designer_UI_exlabel_input_blur(flag)
	{
		var $input = $(this);
		var $label = $(this).parent();
		
		$label.data('callbacks').onEditEnd.call($label);
		
		var value = $input.val();
		if (flag !== false)
		{
			$label.children('.c').text(value);
			$label.data('callbacks').onChange.call($label, value);
		}
		
		$label.children('.c').show();
		$input.remove();
	}
	
	Designer.UI.SectionContainer = function(data)
	{
		var self = this;
		
		if (data == undefined) data = {};
		
		self.$DOM = $('<div class="sectionContainer"></div>');
		initExtraData(self.$DOM, data);
		
		self.$DOM.data('_data', data);
		
		self._data = data;
		self._groups = {};
		self._groupId = 0;
		self._groupCount = 0;
	};
	Designer.UI.SectionContainer.prototype = 
	{
		constructor : Designer,
		
		addGroup: function(group)
		{
			var self = this;
			
			var id = ++self._groupId;
			
			group.appendTo(self.$DOM);
			group._data._id = id;
			group._data._parentContainer = self;
			group.$DOM.attr('data-id', id);
			group.resize();
			
			self._groups[id] = group;
		},
		
		removeGroup: function(id)
		{
			var self = this;
			
			self._groupCount--;
			self._groups[id] = null;
			delete self._groups[id];
		},
		
		destroy: function()
		{
			var self = this;
			
			for (var k in self._groups)
			{
				self._groups[k].destroy();
			}
			
			self.$DOM.remove();
			self.$DOM.removeData('_data');
			self.$DOM = null;
			self._groups = null;
			self._data = null;
			
			delete self.$DOM;
			delete self._data;
			delete self._groups;
			delete self._groupId;
			delete self._groupCount;
		}
	};
	Designer.UI.SectionGroup = function(data)
	{
		var self = this;
		
		if (data == undefined) data = {};
		
		self._groupCount++;
		
		self.$DOM = $('<div class="sectionGroup"><div class="title"></div><div class="view"><div class="content"></div></div></div>');
		initExtraData(self.$DOM, data);
		
		self.$DOM.data('_data', data);
		
		if (data.fullHeight !== true)
			self.$DOM.children('.view').css('height', 0);
		
		self._data = data;
		self._colWidth = 0;
		self._sections = {};
		self._sectionId = 0;
		self._sectionCount = 0;
	}
	Designer.UI.SectionGroup.prototype = 
	{
		constructor : Designer,
		
		addSection : function(section)
		{
			var self = this;
			
			self._sectionCount++;
			
			var id = ++self._sectionId;
			var sectionTitle = section._data.title || 'Section' + id.toString();
			
			var $title = self.$DOM.children('.title');
			var $content = self.$DOM.children('.view').children();
			
			section.appendTo($content);
			section._data._id = id;
			section._data._parentGroup = self;
			section.$DOM.attr('data-id', id);
			
			var titleItem = $('<div class="li"></div>').data('id', id);
			titleItem.html(sectionTitle).click(eh_Designer_UI_sectionGroupTitleClicked).appendTo($title);
			
			if (section._data.maintainWidth !== true) 
				section.$DOM.css('width', self._colWidth);
			
			self._sections[id] = section;
			
			//Only item
			if (self._sectionCount == 1) titleItem.click();
		},
		
		updateSize: function()
		{
			var self = this;
			var $view = self.$DOM.children('.view');
			
			var id = parseInt(self.$DOM.attr('data-current'));
			var $section = $view.find('.section[data-id="' + id + '"]');
			
			$view.css('height', $section.outerHeight(true));
		},
		
		resize: function()
		{
			var self = this;
			
			self._colWidth = self.$DOM.children('.view').width();
			
			if (self._sections.length > 0)
			{
				for (var i in self._sections)
				{
					var curSection = self._sections[i];
					
					if (curSection.data.maintainWidth !== true) 
						curSection.$DOM.css('width', self._colWidth);
				}
			}
		},
		
		appendTo: function($target)
		{
			var self = this;
			self.$DOM.appendTo($target);
			
			if (self._data != undefined && self._data._id != undefined)
				self._data._parentContainer.removeGroup(self._data._id);
		},
		
		removeSection: function(id)
		{
			var self = this;
			
			self._sectionCount--;
			self._sections[id] = null;
			delete self._sections[id];
		},
		
		remove: function()
		{
			var self = this;
			
			if (self._data != undefined && self._data._id != undefined)
				self._data._parentContainer.removeGroup(self._data._id);
			
			self.$DOM.remove();
		},
		
		destroy: function()
		{
			var self = this;
			
			for (var k in self._sections)
			{
				self._sections[k].destroy();
			}
			
			self.remove();
			self.$DOM.removeData('_data');
			self.$DOM = null;
			self._sections = null;
			self._data = null;
			
			delete self.$DOM;
			delete self._data;
			delete self._sections;
			delete self._sectionId;
			delete self._sectionCount;
			delete self._colWidth;
		}
	}
	Designer.UI.Section = function(data)
	{
		var self = this;
		
		if (data == undefined) data = {};
		
		self.$DOM = $('<div class="section"></div>');
		initExtraData(self.$DOM, data);
		self.$DOM.data('_data', data);
		
		if (data.content != undefined) self.$DOM.append(data.content);
		
		self._data = data;
	}
	Designer.UI.Section.prototype = 
	{
		constructor : Designer,
		
		appendTo: function($target)
		{
			var self = this;
			self.$DOM.appendTo($target);
			
			if (self._data != undefined && self._data._id != undefined)
				self._data._parentGroup.removeSection(self._data._id);
		},
		
		remove: function()
		{
			var self = this;
			
			if (self._data != undefined && self._data._id != undefined)
				self._data._parentGroup.removeSection(self._data._id);
			
			self.$DOM.remove();
		},
		
		destroy: function()
		{
			var self = this;
			
			self.remove();
			self.$DOM.removeData('_data');
			self.$DOM = null;
			self._data = null;
			
			delete self._data;
			delete self.$DOM;
		}
	}
	
	/**
	 * (Event Handler : Designer.UI) 选项卡标题单击 
	 */
	function eh_Designer_UI_sectionGroupTitleClicked()
	{
		var $titleItem = $(this), id = $titleItem.data('id');
		var $sectionGroup = $titleItem.parents('.sectionGroup');
		var $view = $sectionGroup.children('.view'), $content = $view.children('.content');
		var $section = $content.children('.section[data-id="' + id + '"]');
		
		$sectionGroup.attr('data-current', id);
		
		$titleItem.siblings().removeClass('actived');
		$titleItem.addClass('actived');
		
		if ($sectionGroup.data('_data').fullHeight !== true)
			$view.css('height', $section.outerHeight(true));
		
		$content.css('left', -$section.position().left);
	}
	
	/**
	 * 根据data初始化元素的id、class、name
	 * 
	 * @private 
	 * @param {Object} $object
	 * @param {Object} data
	 */
	function initExtraData($object, data)
	{
		if (data == undefined) return;
		
		if (data.id != undefined)
			$object.attr('id', data.id);
		
		if (data.class != undefined)
			$object.addClass(data.class);
		
		if (data.name != undefined)
			$object.attr('name', data.name);
		
		//if (data.data != undefined)
		//	for (var key in data.data)
		//		$object.data(key, data.data[key]);
		
		return true;
	}
	
	/**
	 * 创建工具箱
	 * 
	 * @private 
	 * @param {Object} data
	 */
	function createToolbox(data)
	{
		var $toolbox = $('<div></div>');
		initExtraData($toolbox, data);
		
		if (data.tools != undefined)
		for (var k in data.tools)
		{
			var current = data.tools[k];
			var $tool = $('<div class="li"></div>');
			initExtraData($tool, current);
			
			$tool.appendTo($toolbox);
			
			if (current.tooltip != undefined)
			{
				$tool.addClass('tooltip');
				$tool.attr('data-title', current.tooltip);
			}
			
			if (current.icon == true)
				$tool.append('<span class="icon"></span>');
			
			if (current.content != undefined)
				$tool.append(current.content);
		}
		
		//Single select
		Designer.UI.initSingleSelectElements($toolbox.children('.li'));
		
		return $toolbox;
	}
	
	/**
	 * LessCSS 相关
	 * 
	 * @alias Designer.Less 
	 */
	Designer.Less = 
	{
		/**
		 * 将LessCSS编译为设计模式下的CSS代码
		 * 
		 * @alias Designer.Less.compileStage 
		 * @param {Object} unmanagedLessCSS
		 * @param {Object} callback
		 */
		compileStage: function(unmanagedLessCSS, callback)
		{
			unmanagedLessCSS = '.slide_page {' + unmanagedLessCSS + '}';
			
			new(less.Parser)().parse(unmanagedLessCSS, function(e, tree)
			{
				if (e != null)
				{
					alert('CSS Error!');
					return '';
				}
				
				var compiledCSS = tree.toCSS();
				
				if (typeof callback == 'function')
					callback(compiledCSS);
			});
		}
	}
	
	function eh_Designer_Canvas_designarea_keydown(event)
	{
		//Shift
		if (event.keyCode == 16)
			Designer.Canvas.MultiSelecting = true;
		
		Designer_Action_slide_event_dispatch('onKeyDown', event);
	}
	
	function eh_Designer_Canvas_designarea_keyup(event)
	{
		switch(event.keyCode)
		{
			case 16:	//Shift
				Designer.Canvas.MultiSelecting = false;
				break;
		}
		
		Designer_Action_slide_event_dispatch('onKeyUp', event);
	}
	
	/**
	 * 向下取整 
	 */
	function Designer_Canvas_transform_floor(Size)
	{
		return {
			left:	Math.floor(Size.left),
			top:	Math.floor(Size.top),
			width:	Math.floor(Size.width),
			height:	Math.floor(Size.height)
		};
	}
	
	/**
	 * 扩大 
	 */
	function Designer_Canvas_transform_enlarge(Size, value)
	{
		value = value || 0;
		
		return {
			left:	Size.left - value,
			top:	Size.top - value,
			width:	Size.width + value * 2,
			height:	Size.height + value * 2
		}
	}
	
	/**
	 * 从幻灯片坐标变换到舞台坐标 
	 */
	function Designer_Canvas_transform_canvas2stage(Size)
	{
		try
		{
			var scale = Designer.Canvas.canvasScale;
			var ofsObj1 = $('#stage>.slide_container')[0], ofsObj2 = $('#stage .slide_step')[0];
			
			if (Size.width != undefined)
				return {
					left:	Size.left * scale + ofsObj1.offsetLeft + ofsObj2.offsetLeft * scale,
					top:	Size.top * scale + ofsObj1.offsetTop + ofsObj2.offsetTop * scale,
					width:	Size.width * scale,
					height:	Size.height * scale
				};
			else
				return {
					left:	Size.left * scale + ofsObj1.offsetLeft + ofsObj2.offsetLeft * scale,
					top:	Size.top * scale + ofsObj1.offsetTop + ofsObj2.offsetTop * scale
				};
		}
		catch(e)
		{
			return Size;
		}
	}
	
	/**
	 * 从舞台坐标变换到幻灯片坐标 
	 */
	function Designer_Canvas_transform_stage2canvas(Size)
	{
		try
		{
			var scale = Designer.Canvas.canvasScale, scaleD = 1 / scale;
			var ofsObj1 = $('#stage>.slide_container')[0], ofsObj2 = $('#stage .slide_step')[0];
			
			if (Size.width != undefined)
				return {
					left:	(Size.left - ofsObj2.offsetLeft * scale - ofsObj1.offsetLeft) * scaleD,
					top:	(Size.top - ofsObj2.offsetTop * scale - ofsObj1.offsetTop) * scaleD,
					width:	Size.width * scaleD,
					height:	Size.height * scaleD
				};
			else
				return {
					left:	(Size.left - ofsObj2.offsetLeft * scale - ofsObj1.offsetLeft) * scaleD,
					top:	(Size.top - ofsObj2.offsetTop * scale - ofsObj1.offsetTop) * scaleD
				};
		}
		catch(e)
		{
			return Size;
		}
	}
	
	/**
	 * 更新单个选择框坐标 
	 */
	function Designer_Canvas_updateBox(selectObj)
	{
		if (selectObj.$box == undefined) return;
		
		var $design = $('#design');
		if ($design.length == 0) return;
		
		selectObj.$box.css(Designer_Canvas_transform_floor(Designer_Canvas_transform_enlarge(Designer_Canvas_transform_canvas2stage({
			left:	selectObj.$wrapper[0].offsetLeft,
			top:	selectObj.$wrapper[0].offsetTop,
			width:	selectObj.$wrapper[0].offsetWidth,
			height:	selectObj.$wrapper[0].offsetHeight
		}), 1)));
	}
	
	/**
	 * 判断一个元素是否已被选中 
	 */
	function Designer_Canvas_is_selected($target, $wrapper)
	{
		var dCanvas = Designer.Canvas;
		
		//如果在多选模式下，那么只要组被选择了就是被选择了
		if (dCanvas.MultiSelecting)
		{
			for (var k in dCanvas.selected)
				if (dCanvas.selected[k].$wrapper != null && dCanvas.selected[k].$wrapper[0] == $wrapper[0]) return true;
			
			return false;
		}
		
		//单选模式下，判断是否是在编辑同一个内容
		for (var k in dCanvas.selected)
			if (dCanvas.selected[k].$target[0] == $target[0]) return true;
		
		return false;
	}
	
	/**
	 * 判断元素是否是绝对定位 
	 */
	function Designer_Canvas_is_absolute(element)
	{
		var styles = getComputedStyle(element, null)
		var position = styles.getPropertyValue('position');
		
		return (position === 'absolute' || position === 'fixed');
	}
	
	/**
	 * 一个元素是否可编辑 
	 */
	function Designer_Canvas_element_iseditable($element)
	{
		if ($element[0].tagName.toLowerCase() == 'img')
			return false;
		else
			return true;
	}
	
	/**
	 * 使一个元素可编辑 
	 */
	function Designer_Canvas_element_editable($element)
	{
		if (!Designer_Canvas_element_iseditable($element)) return;
		
		var MutationObserver = window.MutationObserver || window.WebKitMutationObserver || window.MozMutationObserver;
		var observer = new MutationObserver(function(mutations) {
			//On tree changed / on content changed
			Designer_Canvas_updateBox($element.data('__selectObj'));
		});
 		
 		observer.observe($element[0], { childList: true, subtree:true, characterData: true });
		
		$element.data('__observer', observer);
		$element.attr('contentEditable', 'true');
		$element.focus();
	}
	
	/**
	 * 是一个元素不可编辑 
	 */
	function Designer_Canvas_element_uneditable($element)
	{
		//停止监听
		if ($element.data('__observer') != undefined)
		{
			$element.data('__observer').disconnect();
			$element.removeData('__observer');
		}
		
		$element.removeAttr('contentEditable');
	}
	
	function eh_Designer_Canvas_select_box_movehandle_dragend()
	{
		Designer_Canvas_updateNode(Designer.UI.DragData.$wrapper);
	}
	
	function eh_Designer_Canvas_select_box_movehandle_dragging(event)
	{
		var dUI = Designer.UI;
		var data = dUI.DragData;
		var scale = Designer.Canvas.canvasScale;
		
		var dX = event.clientX - dUI.DragClientX, dY = event.clientY - dUI.DragClientY;
		var dCX = dX / scale, dCY = dY / scale;
		
		data.$wrapper.css(Designer_Canvas_transform_floor({left: data.posWrapper.left + dCX, top: data.posWrapper.top + dCY}));
		
		Designer_Canvas_updateBox(data);
	}
	
	function eh_Designer_Canvas_select_box_corners_dragging(event)
	{
		var dUI = Designer.UI;
		var data = dUI.DragData;
		var scale = Designer.Canvas.canvasScale;
		
		var dX = event.clientX - dUI.DragClientX, dY = event.clientY - dUI.DragClientY;
		var dCX = dX / scale, dCY = dY / scale;
		
		switch(data.direction)
		{
			case 'b':
				data.$wrapper.css('height', Math.floor(data.size.height + dCY));
				break;
			case 'r':
				data.$wrapper.css('width', Math.floor(data.size.width + dCX));
				break;
			case 't':
				if (data.size.height - dCY < 0) dCY = data.size.height;
				data.$wrapper.css(Designer_Canvas_transform_floor({top: data.size.top + dCY, height: data.size.height - dCY}));
				break;
			case 'l':
				if (data.size.width - dCX < 0) dCX = data.size.width;
				data.$wrapper.css(Designer_Canvas_transform_floor({left: data.size.left + dCX, width: data.size.width - dCX}));
				break;
		}
		
		Designer_Canvas_updateBox(data);
	};
	
	function eh_Designer_Canvas_select_box_movehandle_mousedown(event)
	{
		var selectObj = $(this).closest('.iexd_selectbox').data('__selectObj');
		var $wrapper = selectObj.$wrapper, $target = selectObj.$target;
		
		var dUI = Designer.UI;
		var posWrapper = {left: $wrapper[0].offsetLeft, top: $wrapper[0].offsetTop};
		
		//不可编辑
		Designer_Canvas_element_uneditable($target);
		
		dUI.DragCursor = 'move';
		dUI.DragData = {posWrapper: posWrapper, $wrapper: $wrapper, $box: selectObj.$box};
		dUI.DragEndCallback = eh_Designer_Canvas_select_box_movehandle_dragend;
		dUI.DragCallback = eh_Designer_Canvas_select_box_movehandle_dragging;
		dUI.startDrag($('#stage'), event);
		
		//禁止deselect
		return false;
	}
	
	function eh_Designer_Canvas_select_box_corners_mousedown(event)
	{
		var selectObj = $(this).closest('.iexd_selectbox').data('__selectObj');
		var $wrapper = selectObj.$wrapper;
		var corner = this;
		
		var dUI = Designer.UI;
		var size = {left: $wrapper[0].offsetLeft, top: $wrapper[0].offsetTop, width: $wrapper[0].offsetWidth, height: $wrapper[0].offsetHeight};
		var direction = $(corner).attr('class');
		
		dUI.DragCursor = window.getComputedStyle(corner).getPropertyValue('cursor');
		dUI.DragData = {$wrapper: $wrapper, direction: direction, size: size, $box: selectObj.$box};
		dUI.DragEndCallback = eh_Designer_Canvas_select_box_movehandle_dragend;
		dUI.DragCallback = eh_Designer_Canvas_select_box_corners_dragging;
		dUI.startDrag($('#stage'), event)
		
		return false;
	}
	
	function Designer_Canvas_select_bindListeners($box)
	{
		var $moveHandle = $box.children('.move');
		
		if ($moveHandle.hasClass('moveable'))
			$moveHandle.mousedown(eh_Designer_Canvas_select_box_movehandle_mousedown);
		
		$box.find('.corners>div').mousedown(eh_Designer_Canvas_select_box_corners_mousedown);
	}
	
	function Designer_Canvas_removeNode($node)
	{
		var dCanvas = Designer.Canvas;
		
		dCanvas.$docContent.find('#' + dCanvas.currentSlideId + ' [data-iexd-id="' + $node.attr('data-iexd-id') + '"]').remove();
		$('#slide_list #' + dCanvas.currentSlideId + ' [data-iexd-id="' + $node.attr('data-iexd-id') + '"]').remove();
	}
	
	function Designer_Canvas_appendNode($node, path)
	{
		var dCanvas = Designer.Canvas;
		
		dCanvas.$docContent.find('#' + dCanvas.currentSlideId + path).append($node.clone());
		$('#slide_list #' + dCanvas.currentSlideId + path).append($node.clone());
	}
	
	function Designer_Canvas_updateNode($node)
	{
		var dCanvas = Designer.Canvas;
		
		dCanvas.$docContent.find('#' + dCanvas.currentSlideId + ' [data-iexd-id="' + $node.attr('data-iexd-id') + '"]').replaceWith($node.clone());
		$('#slide_list #' + dCanvas.currentSlideId + ' [data-iexd-id="' + $node.attr('data-iexd-id') + '"]').replaceWith($node.clone());
	}
	
	/**
	 * 为元素添加选择框 
	 */
	function Designer_Canvas_select($target, $wrapper, multi_selecting)
	{
		var is_select_canvas = ($wrapper == null);
		multi_selecting = multi_selecting || false;
		
		var dCanvas = Designer.Canvas;
		var newSelectObj = {$target: $target, $wrapper: $wrapper};
		
		Designer_Canvas_hottrack_remove();
		
		//==========================================
		//附加信息
		$target.data('__selectObj', newSelectObj);
		
		if ($wrapper != null)
			$wrapper.data('__selectObj', newSelectObj);
		//==========================================
		
		dCanvas.selected.push(newSelectObj);
		Designer.Events.trigger('ElementSelected');
		
		//对于画布来说不需要选择框
		if (is_select_canvas) return;
		
		//添加可编辑标记
		if (Designer_Canvas_element_iseditable($target))
			$target.addClass('iexd_editable');
		
		//移除empty标记
		$target.removeClass('iexd_empty');
		
		//附加selectbox
		newSelectObj.$box = $('<div class="iexd_selectbox"><div class="move"></div></div>').appendTo('#stage');
		newSelectObj.$box.data('__selectObj', newSelectObj);
		Designer_Canvas_updateBox(newSelectObj);
		
		if (Designer_Canvas_is_absolute($wrapper[0]))
		{
			//可移动
			var e = newSelectObj.$box.children('.move');
			e.addClass('moveable');
		}
		else
		{
			var e = newSelectObj.$box.children('.move');
			e.addClass('not_moveable').html('不可移动(非绝对定位)');
			
			Designer.c['d'].newDelayer(10, function(){e.addClass('hide');});
		}
		
		//Corners
		newSelectObj.$box.append('<div class="corners"><div class="lt"></div><div class="rt"></div><div class="lb"></div><div class="rb"></div><div class="l"></div><div class="t"></div><div class="r"></div><div class="b"></div></div>');
		
		//添加事件监听
		Designer_Canvas_select_bindListeners(newSelectObj.$box);
		
		if (dCanvas.MultiSelecting || multi_selecting)
		{
			for (var k in dCanvas.selected)
				Designer_Canvas_element_uneditable(dCanvas.selected[k].$target);
			
			return;
		}
	}
	
	function deselectSelections()
	{
		//Deselect text
		var sel = window.getSelection();
		if (sel.rangeCount == 0) return;
		
		var range = sel.getRangeAt(0);
		range.collapse(true);
	}
	
	/**
	 * 为元素移除选择框 
	 */
	function Designer_Canvas_deselect($wrapper)
	{
		deselectSelections();
		
		var dCanvas = Designer.Canvas;
		if (dCanvas.selected == null) return;
		
		for (var k in dCanvas.selected)
		{
			if (dCanvas.selected[k].$wrapper != null && dCanvas.selected[k].$wrapper[0] == $wrapper[0])
			{
				Designer_Canvas_deselect_internel(parseInt(k), true);
				Designer.Events.trigger('ElementDeselected');
				return;
			}
		}
	}
	
	/**
	 * 为所有元素移除选择框 
	 */
	function Designer_Canvas_deselectAll()
	{
		deselectSelections();
		
		var dCanvas = Designer.Canvas;
		if (dCanvas.selected == null) return;
		
		for (var k in dCanvas.selected)
			Designer_Canvas_deselect_internel(parseInt(k), false);
		
		dCanvas.selected = [];
		Designer.Events.trigger('ElementDeselected');
	}
	
	/**
	 * 为除了指定元素外的元素移除选择框 
	 */
	function Designer_Canvas_deselectOther($wrapper)
	{
		deselectSelections();
		
		var dCanvas = Designer.Canvas;
		if (dCanvas.selected == null) return;
		
		var currentId;
		
		for (var k in dCanvas.selected)
		{
			if (dCanvas.selected[k].$wrapper != null && dCanvas.selected[k].$wrapper[0] == $wrapper[0])
			{
				currentId = parseInt(k);
				
				for (var i = 0; i < currentId; i++)
					Designer_Canvas_deselect_internel(i, false);
				
				for (var i = currentId+1; i < dCanvas.selected.length; i++)
					Designer_Canvas_deselect_internel(i, false);
					
				dCanvas.selected = [dCanvas.selected[currentId]];
				Designer.Events.trigger('ElementDeselected');
				
				return;
			}
		}
	}
	
	/**
	 * 取消选择核心函数 
	 */
	function Designer_Canvas_deselect_internel(k, splice)
	{
		var dCanvas = Designer.Canvas;
		if (dCanvas.selected == null) return;
		if (dCanvas.selected[k] == undefined) return;
		
		if (splice == undefined) splice = true;
		
		var cur = dCanvas.selected[k];
		if (cur.$box != undefined)
			cur.$box.remove();
		
		//=========================================
		//移除信息
		if (cur.$wrapper != null)
			cur.$wrapper.removeData('__selectObj');
		cur.$target.removeData('__selectObj');
		//=========================================
		
		Designer_Canvas_element_uneditable(cur.$target);
		cur.$target.removeClass('iexd_editable');
		
		//如果内容是空白，则添加虚线框
		if (cur.$target.text().replace(/\s/g, '').length == 0)
			cur.$target.addClass('iexd_empty');
		
		//更新节点内容
		if (cur.$wrapper != null)
			Designer_Canvas_updateNode(cur.$target);
		
		dCanvas.selected[k] = null;
		
		if (splice)
			dCanvas.selected.splice(k, 1);
	}
	
	function cb_Designer_Canvas_boundselect_dragEnd_IsIntersect(wrapper, bound)
	{
		if (wrapper.offsetLeft + wrapper.offsetWidth <= bound.left) return false;
		if (wrapper.offsetLeft >= bound.left + bound.width) return false;
		if (wrapper.offsetTop + wrapper.offsetHeight <= bound.top) return false;
		if (wrapper.offsetTop >= bound.top + bound.height) return false;
		
		return true;
	}
	function cb_Designer_Canvas_boundselect_dragEnd_IsInList(wrapper, objList)
	{
		for (var k in objList)
			if (objList[k].wrapper == wrapper) return true;
		
		return false;
	}
		
	function cb_Designer_Canvas_boundselect_select_dragEnd()
	{
		var dCanvas = Designer.Canvas;
		var bound = Designer_Canvas_transform_floor(Designer_Canvas_transform_stage2canvas(dCanvas.Bound));
		
		dCanvas.$BoundBox.remove();
		dCanvas.$BoundBox = null;
		
		var objList = [];
		//计算框选元素
		$('#design .slide_step [data-iexd-edit]').each(function()
		{
			var target = this, $wrapper = $(this).closest('[data-iexd-wrap]');
			if ($wrapper.length == 0 || !cb_Designer_Canvas_boundselect_dragEnd_IsIntersect($wrapper[0], bound)) return;
			
			//是否在区域内
			if (!cb_Designer_Canvas_boundselect_dragEnd_IsInList($wrapper[0], objList))
				objList.push({target:target, wrapper:$wrapper[0]});
		});
		
		for (var i = 0; i < objList.length; i++)
			Designer_Canvas_select($(objList[i].target), $(objList[i].wrapper), true);
	}
	
	function cb_Designer_Canvas_boundselect_text_dragEnd()
	{
		var dCanvas = Designer.Canvas;
		var bound = Designer_Canvas_transform_floor(Designer_Canvas_transform_stage2canvas(dCanvas.Bound));
		
		dCanvas.$BoundBox.remove();
		dCanvas.$BoundBox = null;
		
		if (bound.height < 20)
			bound.height = 30;
		
		if (bound.width < 20)
			bound.width = 100;
		
		var $newElement = $('<div style="position:absolute;" data-iexd-edit data-iexd-wrap></div>').css(bound);
		Designer_Canvas_build_node($newElement, true);
		
		var insertPath = '';
		//if ($('#design .f_body>.slide_step' + insertPath).length == 0)
		//	insertPath = '';
		
		$newElement.appendTo($('#design .f_body>.slide_step' + insertPath));
		Designer_Canvas_appendNode($newElement, insertPath);
		
		Designer_Canvas_select($newElement, $newElement);
		Designer_Canvas_element_editable($newElement);
		
		$('#tools .li[name="select"]').click();
	}
	
	function cb_Designer_Canvas_boundselect_dragging(event)
	{
		var dUI = Designer.UI, dCanvas = Designer.Canvas;
		var dX = event.clientX - dUI.DragClientX, dY = event.clientY - dUI.DragClientY;
		var bound = {};
		
		var posX = dCanvas.BoundOfs.left, posY = dCanvas.BoundOfs.top;
		
		if (dX > 0)
		{
			bound.left = dUI.DragClientX - posX; bound.width = dX;
		}
		else
		{
			bound.left = event.clientX - posX; bound.width = -dX;
		}
		if (dY > 0)
		{
			bound.top = dUI.DragClientY - posY; bound.height = dY;
		}
		else
		{
			bound.top = event.clientY - posY; bound.height = -dY;
		}
		
		dCanvas.Bound = bound;
		dCanvas.$BoundBox.css(bound);
	}
	
	function Designer_Canvas_hottrack_remove()
	{
		var dCanvas = Designer.Canvas;
		
		if (dCanvas.hottrack != null)
		{
			dCanvas.hottrack.$box.remove();
			dCanvas.hottrack = null;
		}
	}
	
	/**
	 * 激活hottrack选择框
	 */
	function eh_Designer_Canvas_select_mousemove(e, event)
	{
		var dCanvas = Designer.Canvas;
		
		//仍然是当前元素 则退出
		if (dCanvas.hottrack != null && e == dCanvas.hottrack.$wrapper[0]) return;
		
		var $target = $(e);
		var $wrapper = $(Designer_Canvas_get_target_wrapper(e));
		
		Designer_Canvas_hottrack_remove();
		
		if (e == null) return;
		if (Designer_Canvas_is_selected($target, $wrapper)) return;
		
		var selectObj = {$wrapper: $wrapper, $box: $('<div class="iexd_hottrackbox"></div>').appendTo('#stage')};
		Designer_Canvas_updateBox(selectObj);
		dCanvas.hottrack = selectObj;
	}	
		
	/**
	 * 画布上选中某个元素
	 * 
	 * @param {Object} e
	 * @param {Object} event
	 */
	function eh_Designer_Canvas_select_mousedown(e, event)
	{
		var dCanvas = Designer.Canvas;
		var $e = null, $wrap = null, is_select_canvas;
		var flag = false;
		
		//Slide
		if (e == null)
		{
			$e = $('#stage .slide_step:eq(0)');
			$wrap = null;
			is_select_canvas = true;
		}
		else
		{
			$e = $(e);
			$wrap = $(Designer_Canvas_get_target_wrapper(e));
			is_select_canvas = false;
		}
		
		if (is_select_canvas)
		{
			if (dCanvas.MultiSelecting) return;	//如果在多选模式下 点击了画布 则不作任何处理
			
			Designer_Canvas_deselectAll();
			Designer_Canvas_select($e, $wrap);
			
			//选择区域
			var dUI = Designer.UI;
			dUI.DragCursor = 'default';
			
			var $stage = $('#stage');
			
			if (dCanvas.$BoundBox != null)
				dCanvas.$BoundBox.remove();
			
			dCanvas.$BoundBox = $('<div class="iexd_boundbox" style="left:-10px;top:-10px;"></div>').appendTo('#stage');
			dCanvas.BoundOfs = $stage.offset();
			
			dUI.DragEndCallback = cb_Designer_Canvas_boundselect_select_dragEnd;
			dUI.DragCallback = cb_Designer_Canvas_boundselect_dragging;
			dUI.startDrag($stage, event);
			
			return;
		}
		
		//当前项已经选择
		if (Designer_Canvas_is_selected($e, $wrap))
		{
			if (dCanvas.MultiSelecting)		//多选中是撤销当前选择的
			{
				Designer_Canvas_deselect($wrap);
				return;
			}
			
			//如果已经选择多个，则取消其它选择
			if (dCanvas.selected.length > 1)
				Designer_Canvas_deselectOther($wrap);
			
			//当前选择的项已经选择，则使它可编辑
			Designer_Canvas_element_editable($e);
			
			return;
		}
		
		if (!dCanvas.MultiSelecting)
			Designer_Canvas_deselectAll();
		
		Designer_Canvas_select($e, $wrap);
	}
	
	function eh_Designer_Canvas_text_mousedown(e, event)
	{
		var dCanvas = Designer.Canvas;
		var dUI = Designer.UI;
		dUI.DragCursor = 'default';
		
		var $stage = $('#stage');
		
		if (dCanvas.$BoundBox != null)
			dCanvas.$BoundBox.remove();
		
		dCanvas.$BoundBox = $('<div class="iexd_draw_boundbox" style="left:-10px;top:-10px;"></div>').appendTo('#stage');
		dCanvas.BoundOfs = $stage.offset();
		
		dUI.DragEndCallback = cb_Designer_Canvas_boundselect_text_dragEnd;
		dUI.DragCallback = cb_Designer_Canvas_boundselect_dragging;
		dUI.startDrag($stage, event);
	}
	
	//Delete
	function eh_Designer_Canvas_delete(event)
	{
		var dCanvas = Designer.Canvas;
		var selected = dCanvas.selected;
		if (selected.length == 0) return;
		
		var tag;
		var deleteNodes = [];
		
		for (var i in selected)
		{
			if (selected[i].$wrapper == null) continue;	//选中画布
			if (selected[i].$target.attr('contentEditable') != undefined) continue;	//编辑中忽略
			
			//如果是标题或者section，则只清空
			tag = selected[i].$wrapper[0].tagName.toLowerCase();
			if (tag == 'header' || tag == 'section')
			{
				selected[i].$wrapper.empty().addClass('iexd_empty iexd_tag_deleted');
				continue;
			}
			
			deleteNodes.push(selected[i].$wrapper);
		}
		
		for (var i in deleteNodes)
		{
			Designer_Canvas_removeNode(deleteNodes[i]);
			Designer_Canvas_deselect(deleteNodes[i]);
			deleteNodes[i].remove();
		}
	}
	
	/**
	 * 全局画布操作相关
	 * 
	 * @alias Designer.Canvas 
	 */
	Designer.Canvas = 
	{
		$docIFrame: null,
		$docContent: null,
		$style: null,
		currentSlideId: null,
		currentTool: null,
		lessStyle: null,
		SlideCount: 0,
		
		selected: null,
		MultiSelecting: false,
		canvasScale: 1,
		
		hottrack: null,
		
		$BoundBox: null,
		BoundOfs: null,
		Bound: null,
		
		/**
		 * 清空/初始化
		 * 
		 * @alias Designer.Canvas.clear 
		 */
		clear: function()
		{
			var dCanvas = Designer.Canvas;
			
			if (dCanvas.$docIFrame != null)
				dCanvas.$docIFrame.remove();
			
			if (dCanvas.$style != null)
				dCanvas.$style.remove();
			
			if (dCanvas.hottrack != null)
				dCanvas.hottrack.$box.remove();
			
			dCanvas.currentTool = null;
			dCanvas.currentSlideId = null;
			dCanvas.hottrack = null;
			dCanvas.MultiSelecting = false;
			dCanvas.selected = [];
			dCanvas.SlideCount = 0;
			dCanvas.lessStyle = $('#css_template').html();
			dCanvas.$style = $('<style type="text/css" media="screen"></style>').appendTo('head');
			
			Designer.Less.compileStage(dCanvas.lessStyle, function(css)
			{
				dCanvas.$style.html(css);
			});
			
			dCanvas.$docIFrame = $('<iframe id="backgroundIFrame"></iframe>').appendTo('body');
			
			var doc = dCanvas.$docIFrame[0].contentWindow.document;
			doc.open();
			doc.write('<html><head><meta http-equiv="Content-Type" content="text/html; charset=utf-8" /></head><body><div id="impressEx"></div></body></html>');
			doc.close();
			
			dCanvas.$docContent = dCanvas.$docIFrame.contents();
		},
		
		/**
		 * 为设计区域绑定事件
		 * 
		 * @alias Designer.Canvas.bindEventListener 
		 */
		bindEventListener: function()
		{
			$(document).bind('keydown', 'ctrl+s', function(event){Designer.Action.save();event.preventDefault();});
			$(document).keydown(eh_Designer_Canvas_designarea_keydown).keyup(eh_Designer_Canvas_designarea_keyup);
		},
		
		/**
		 * 更新选择框位置
		 * 
		 * @alias Designer.Canvas.updateSelectBox 
		 */
		updateSelectBox: function()
		{
			var dCanvas = Designer.Canvas;
			if (dCanvas.selected == null) return;
			
			for (var k in dCanvas.selected)
				Designer_Canvas_updateBox(dCanvas.selected[k]);
			
			if (dCanvas.hottrack != null)
				Designer_Canvas_updateBox(dCanvas.hottrack);
		},
		
		onMouseMove_select: function(e, event)
		{
			eh_Designer_Canvas_select_mousemove(e, event);
		},
		onMouseDown_select: function(e, event)
		{
			eh_Designer_Canvas_select_mousedown(e, event);
		},
		onKeyDown_select: function(event)
		{
			switch(event.which)
			{
				case 27:	//ESC
					Designer_Canvas_deselectAll();
					break;
				case 46:	//Delete
					return eh_Designer_Canvas_delete(event);
					break;
			}
		},
		onMouseMove_text: function(e, event)
		{
			//Do nothing	
		},
		onSwitchIn_text: function()
		{
			Designer_Canvas_deselectAll();
		},
		onMouseDown_text: function(e, event)
		{
			eh_Designer_Canvas_text_mousedown(e, event);
		}
	}
	
	function eh_Designer_Action_slide_list_click()
	{
		if ($(this).hasClass('selected')) return;
		
		Designer_Canvas_deselectAll();
		
		$('#slide_list>.slide.selected').removeClass('selected');
		$(this).addClass('selected');
		
		var dCanvas = Designer.Canvas;
		
		dCanvas.currentSlideId = $(this).find('.slide_step').attr('id');
		$('#design>.f_body').empty().append(dCanvas.$docContent.find('#' + dCanvas.currentSlideId).clone());
	}
	
	function get_target_wrapper(element)
	{
		if (!$(element).hasClass('wrapchild'))
			return element;
		
		return $(element).closest('.wrapper');
	}
	
	function calculate_target_element(element)
	{
		switch(element.tagName.toLowerCase())
		{
			case 'img':			//图片
			case 'header':
			case 'section':
				return element;
				break;
		}
		
		var $e = $(element);
		var $closest = $e.closest('header, section, div.float');
		if ($closest.length == 0)
			return null;
		else
			return $closest[0];
	}
	
	function Designer_Canvas_get_target_wrapper(editable_element)
	{
		if (editable_element == null)
			return null;
		else
			return $(editable_element).closest('[data-iexd-wrap]')[0];
	}
	
	function Designer_Canvas_get_target_element(element)
	{
		var $e = $(element).closest('[data-iexd-edit]');
		
		if ($e.length == 0)
			return null;
		else
			return $e[0];
	}
	
	function Designer_Action_slide_event_dispatch(prefix)
	{
		var args = Array.prototype.slice.call(arguments);
		args.shift();
		
		var callTarget = Designer.Canvas[prefix + '_' + Designer.Canvas.currentTool];
		if (callTarget == undefined || typeof callTarget != 'function') return;
		
		callTarget.apply(window, args);
	}
	
	function eh_Designer_Action_slide_mousemove(event)
	{
		var e = Designer_Canvas_get_target_element(document.elementFromPoint(event.clientX, event.clientY));
		Designer_Action_slide_event_dispatch('onMouseMove', e, event);
	}
	
	function eh_Designer_Action_slide_mousedown(event)
	{
		var e = Designer_Canvas_get_target_element(document.elementFromPoint(event.clientX, event.clientY));
		Designer_Action_slide_event_dispatch('onMouseDown', e, event);
	}
	
	function eh_Designer_Action_slide_mouseup(event)
	{
		
	}
	
	function Designer_Canvas_build_node($node, assignSelfId)
	{
		$node.find('*').each(function()
		{
			var node = $(this);
			node.attr('data-iexd-id', Designer.Id.random());
		});
		
		if (assignSelfId == true)
			$node.attr('data-iexd-id', Designer.Id.random());
		
		return $node;
	}
	
	function Designer_Canvas_build_slide(HTML)
	{
		var $slide = $(HTML);
		$slide = Designer_Canvas_build_node($slide);
		
		//寻找叶子节点
		$slide.find(':not(:has(*))').each(function()
		{
			var node = this;
			var element = calculate_target_element(node), wrapper = get_target_wrapper(element);
			
			$(element).attr('data-iexd-edit', '');
			$(wrapper).attr('data-iexd-wrap', '');
		});
		
		return $slide;
	}
	
	function eh_Designer_UI_section_properties_update(event)
	{
		var context = Designer.c['d'];
		context.destroyDelayer('update_section_properties');
		context.newDelayer('update_section_properties', 0, cb_Designer_UI_section_properties_update);
	}
	
	function cb_Designer_UI_section_properties_update()
	{	
		var section = Designer.UI.SectionProperties;
		var selected = Designer.Canvas.selected;
		
		//自动显示/隐藏
		if (selected.length == 1 && selected[0].$wrapper != null)
			Designer.UI.RightSidebar.$DOM.addClass('active');
		else
			Designer.UI.RightSidebar.$DOM.removeClass('active');
		
		if (selected.length == 1)
		{
			section.$DOM.find('.position input').removeAttr('disabled');
		}
		else
		{
			section.$DOM.find('.position input').attr('disabled','');
		}
	}
	
	function eh_Designer_UI_section_properties_exlabel_onEditBegin()
	{
		$(this).closest('.sectionContainer').addClass('active');
	}
	
	function eh_Designer_UI_section_properties_exlabel_onEditEnd()
	{
		$(this).closest('.sectionContainer').removeClass('active');
	}
	
	function eh_Designer_UI_section_properties_exlabel_onChange(value)
	{
		switch($(this).attr('name'))
		{
			case 'exlabel_id':
				break;
			case 'exlabel_x':
				break;
			case 'exlabel_y':
				break;
			case 'exlabel_w':
				break;
			case 'exlabel_h':
				break;
		}
		
		console.log($(this).attr('name'), value);
	}
	
	
	
	/**
	 * @alias Designer.Action 
	 */
	Designer.Action = 
	{
		/**
		 * 创建新的一页
		 * 
		 * @alias Designer.Action.newSlidePage 
		 * @param {Object} insertBefore		如果指定，则表示插入在哪一页之前
		 */
		newSlidePage: function(insertBefore)
		{
			if (insertBefore != undefined)
			{
				alert('insertBefore failed');
				return;
			}
			
			//设计视图中没有slide
			if ($('#design').length == 0)
			{
				var $design = $('<div id="design" class="slide_page"><div class="f_body"></div></div>').appendTo('#stage>.slide_container');
				Designer.c['d'].newDelayer(10, function(){$design.addClass('show');});
			}
			
			var dG = Designer.Canvas, slideId = ++dG.SlideCount;
			var HTML = '<div class="slide_step" id="slide_' + slideId + '"><header>在此输入标题</header><section>slide' + slideId + '</section></div>';
			var $new = Designer_Canvas_build_slide(HTML);
			
			var $slideSmall = $('<div class="slide"><div class="number"></div><div class="click_layer"></div><div class="slide_container"><div class="content slide_page"><div class="f_body"></div></div></div></div>');
			
			dG.$docContent.find('#impressEx').append($new.clone());
			$new.appendTo($slideSmall.find('.f_body'));
			
			$slideSmall.click(eh_Designer_Action_slide_list_click).click();
			$slideSmall.appendTo('#slide_list');
			onSidebarResizeEnd();
			
			//重新计算数量
			$('#slide_list>.slide').each(function(idx)
			{
				$(this).children('.number').html($(this).find('.slide_step').attr('id') + ' [' + (idx+1).toString() + ']');
			});
		},
		
		save: function()
		{
			alert('保存还没实现呢 ╮(╯▽╰)╭');
		},
		
		/**
		 * 创建新演示文档
		 * 
		 * @alias Designer.Action.createNew 
		 */
		createNew: function()
		{
			var dUI = Designer.UI;
			
			var $body = $('#container>.body').empty();
			Designer.Canvas.clear();
			
			$('#codemode').show();
			
			//Sections
			if (dUI.SectionSlidelist != undefined) dUI.SectionSlidelist.destroy();
			var sectionSlidelist = dUI.SectionSlidelist = new dUI.Section({
				maintainWidth: true,
				title:		'幻灯片',
				content:	'<div id="slide_list"></div>'
			});
			
			function createSectionPropertiesLine(line_class, line_caption, exlabel_name, exlabel_def, exlabel_set)
			{
				var s = '<div class="' + line_class + '"><div class="d">' + line_caption + '</div><div class="exlabel" name="' + exlabel_name + '"';
				
				if (exlabel_set != undefined)
					s += ' data-set="' + exlabel_set + '"';
				
				s += '><div class="c">' + exlabel_def + '</div></div></div>';
				
				return s;
			}
			
			if (dUI.SectionProperties != undefined) dUI.SectionProperties.destroy();
			var sectionProperties = dUI.SectionProperties = new dUI.Section({
				title:	'属性',
				name:	'properties',
				content:
				'<div class="position">' +
				createSectionPropertiesLine('line', 'ID', 'exlabel_id', 'id') + 
				createSectionPropertiesLine('col', 'X', 'exlabel_x', '0', '{\'floor\':true}') + 
				createSectionPropertiesLine('col', 'Y', 'exlabel_y', '0', '{\'floor\':true}') + 
				createSectionPropertiesLine('col', '长', 'exlabel_w', '0', '{\'floor\':true}') + 
				createSectionPropertiesLine('col', '宽', 'exlabel_h', '0', '{\'floor\':true}') +
				'</div>'
			});
			
			Designer.UI.initExtendLabel(sectionProperties.$DOM.find('.exlabel'),
			{
				onEditBegin: eh_Designer_UI_section_properties_exlabel_onEditBegin,
				onEditEnd: eh_Designer_UI_section_properties_exlabel_onEditEnd,
				onChange: eh_Designer_UI_section_properties_exlabel_onChange
			});
			
			Designer.Events.hook('ElementSelected', 'SectionProperties', eh_Designer_UI_section_properties_update);
			Designer.Events.hook('ElementDeselected', 'SectionProperties', eh_Designer_UI_section_properties_update);
			
			if (dUI.SectionGProperties != undefined) dUI.SectionGProperties.destroy();
			var sectionGProperties = dUI.SectionGProperties = new dUI.Section({
				title:		'幻灯片属性',
				content:	''
			});
			
			//========= Left sidebar ============
			
			if (dUI.LeftSidebar != undefined)
				dUI.LeftSidebar.destroy();
			
			var leftSidebar = dUI.LeftSidebar = new dUI.SectionContainer({
				id:		'left_sidebar',
				class:	'resizer1',
				data:
				{
					'min-width':	100,
					'max-width':	400
				}
			});
			leftSidebar.$DOM.appendTo($body);
			
			var group = new dUI.SectionGroup({fullHeight: true});
			leftSidebar.addGroup(group);
			group.addSection(sectionSlidelist);
			
			//========= Main ============
			
			var $main = $('<div id="main" class="section resizer2" data-min-width="300"><div><div id="stage"><div class="slide_container"><div class="ruler rX"><span></span></div><div class="ruler rY"><span></span></div></div></div></div></div>');
			
			//创建工具箱
			var $tools = createToolbox(
			{
				id:		'tools',
				tools: 
				[
					{name:'select', tooltip:'选择', icon:true},
					{name:'text', tooltip:'文本框', icon:true},
					{name:'image', tooltip:'插入图片', icon:true}
				]
			});
			
			$tools.children('.li').click(function()
			{
				Designer.Canvas.currentTool = $(this).attr('name');
				
				switch(Designer.Canvas.currentTool)
				{
					case 'select':
						$('#stage').attr('data-cursor', 'default');
						break;
					case 'text':
						$('#stage').attr('data-cursor', 'crosshair');
						break;
				}
				
				Designer_Action_slide_event_dispatch('onSwitchIn');
			});
			$tools.insertBefore($main.find('#stage'));
			$tools.find('[name="select"]').click();
			
			$main.appendTo($body);
			$('#stage')
			.mousemove(eh_Designer_Action_slide_mousemove)
			.mousedown(eh_Designer_Action_slide_mousedown)
			.mouseup(eh_Designer_Action_slide_mouseup);
			
			//dUI.bindContextMenu($('#stage'), {$menu: $('#menu_stage'), callback: mh_menu_stage});
			
			//========= Right sidebar ============
			
			if (dUI.RightSidebar != undefined)
				dUI.RightSidebar.destroy();
			
			var rightSidebar = dUI.RightSidebar = new dUI.SectionContainer(
			{
				id:		'right_sidebar'
			});
			rightSidebar.$DOM.appendTo($main);
			
			var group = new dUI.SectionGroup();
			rightSidebar.addGroup(group);
			group.addSection(sectionProperties);
			group.addSection(sectionGProperties);
			
			//=================================
			
			Designer.UI.panelResizable($('#container>.body'), 
			{
				horizontal: true,
				realtime: false,
				onResizing: onSidebarResizing,
				onStart: onSidebarResizeStart,
				onEnd: onSidebarResizeEnd
			});
			onSidebarResizeEnd();
			
			Designer.Action.newSlidePage();
		}
	};
	
	/**
	 * (Event Handler : Menu) 画布上按钮回调 
	 * @param {Object} action
	 * @param {Object} el
	 */
	function mh_menu_stage(action, el)
	{
		switch(action)
		{
			case 'newpage':
				Designer.Action.newSlidePage();
				break;
		}
	}
	
	//=================== Main =======================
	
	
	function resizeSlidePage($slide, container_width, container_height, $resizer, resizeFactor)
	{
		resizeFactor = resizeFactor || 0.9;
		
		var fX = container_width / SLIDE_WIDTH, fY = container_height / SLIDE_HEIGHT;
		var factor = fX*resizeFactor;
		var transform = 'scale(' + factor + ')';
		
		if ((container_height != undefined && fY < fX) || container_width == undefined)
		{
			factor = fY*resizeFactor;
			transform = 'scale(' + factor + ')';
		}
		
		$slide.each(function()
		{
			$(this).css(PrefixFree.Prefix + 'Transform', transform);
			
			if ($resizer != undefined)
			{
				$resizer.width(SLIDE_WIDTH * factor);
				$resizer.height(SLIDE_HEIGHT * factor);
			}
		});
		
		return factor;
	}
	
	function onSidebarResizing(event, v)
	{
		var cw = ($('#container .body').width() - v - Designer.UI.ResizeSize), ch = $('#stage').height();
		var fX = cw / SLIDE_WIDTH, fY = ch / SLIDE_HEIGHT;
		
		var factor = fX;
		if (fY < fX) factor = fY;
		factor = factor * 0.9;
		
		$('#stage .ruler.rX span').html(Math.floor(factor * 100) + '% - ' + Math.floor(SLIDE_WIDTH*factor) + 'px / ' + SLIDE_WIDTH + 'px');
		$('#stage .ruler.rY span').html(Math.floor(SLIDE_HEIGHT*factor) + 'px / ' + SLIDE_HEIGHT + 'px');
	}
	
	function onSidebarResizeStart()
	{
		$('#stage .ruler').addClass('show');
	}
	
	function onSidebarResizeEnd()
	{
		var $c = $('#slide_list');
		if ($c.length == 0) return;
		
		resizeSlidePage($c.find('.slide_page'), $c.width(), undefined, $c.find('.slide_container'), 1);
		
		$c = $('#stage');
		$('#stage .ruler').removeClass('show');
		var factor = resizeSlidePage($c.find('.slide_page'), $c.width(), $c.height(), $c.find('.slide_container'));
		
		Designer.Canvas.canvasScale = factor;
		Designer.Canvas.updateSelectBox();
	}
	
	//============== Initializers ==================
	
	function initCodeMode()
	{
		var codearea_shown = false;
		
		var $codemodeItem = $('#codemode>.li');
		Designer.UI.initMultipleSelectElements($codemodeItem);
		
		$codemodeItem.click(function()
		{
			var $codebody = $('#container>.code_body');
			
			var target = $codebody.children('.c_' + $(this).attr('name'));
			if ($(this).hasClass('selected'))
				target.addClass('show');
			else
				target.removeClass('show');
			
			$codebody.children().removeClass('clearmargin');
			
			var l = $('#codemode>.li.selected').length;
			if (l == 0)
			{
				codearea_shown = false;
				$codebody.hide();
				$('#codemode>.commands').hide();
				$('#container>.body').show();
				
				return;
			}
			
			$codebody.children('.show:last').addClass('clearmargin');
			
			if (l == 3)
				$codebody.children().css('width','32%')
			else if (l == 2)
				$codebody.children().css('width','49%')
			else
				$codebody.children().css('width','100%')
			
			if (!codearea_shown)
			{
				$codebody.show();
				$('#codemode>.commands').show();
				$('#container>.body').hide();
				codearea_shown = true;
			}
		});
	}
	
	function initActions()
	{
		$('#container>.top>.commands>.li').click(function()
		{
			if ($(this).hasClass('disabled')) return;
			
			switch($(this).data('action'))
			{
				case 'create':
					Designer.Action.createNew();
					break;
				case 'save':
					Designer.Action.save();
					break;
				case 'preview':
					Designer.Action.preview();
					break;
			}
		});
	}
	
	$(document).ready(function()
	{
		initCodeMode();
		initActions();
		
		Designer.Canvas.bindEventListener();
		$(window).resize(onSidebarResizeEnd);
		
		window.onbeforeunload = function(event)
		{
			return '您的更改还没有保存。';
		}
		
		$('#loading').fadeOut('medium', function(){$(this).remove();});
	});
})(window);